from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.field_type import FieldType
from ..models.field_value import FieldValue
from ..types import UNSET, Unset

T = TypeVar("T", bound="Field")


@attr.s(auto_attribs=True, repr=False)
class Field:
    """  """

    _value: Union[None, str, bool, float, FieldValue, List[str], UnknownType]
    _display_value: Union[Unset, None, str] = UNSET
    _is_multi: Union[Unset, bool] = UNSET
    _text_value: Union[Unset, None, str] = UNSET
    _type: Union[Unset, FieldType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("display_value={}".format(repr(self._display_value)))
        fields.append("is_multi={}".format(repr(self._is_multi)))
        fields.append("text_value={}".format(repr(self._text_value)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("value={}".format(repr(self._value)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Field({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        display_value = self._display_value
        is_multi = self._is_multi
        text_value = self._text_value
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        value: Union[None, str, bool, float, Dict[str, Any], List[Any]]
        if isinstance(self._value, Unset):
            value = UNSET
        if self._value is None:
            value = None
        elif isinstance(self._value, UnknownType):
            value = self._value.value
        elif isinstance(self._value, FieldValue):
            value = self._value.to_dict()

        elif isinstance(self._value, list):
            value = self._value

        else:
            value = self._value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if display_value is not UNSET:
            field_dict["displayValue"] = display_value
        if is_multi is not UNSET:
            field_dict["isMulti"] = is_multi
        if text_value is not UNSET:
            field_dict["textValue"] = text_value
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_display_value() -> Union[Unset, None, str]:
            display_value = d.pop("displayValue")
            return display_value

        try:
            display_value = get_display_value()
        except KeyError:
            if strict:
                raise
            display_value = cast(Union[Unset, None, str], UNSET)

        def get_is_multi() -> Union[Unset, bool]:
            is_multi = d.pop("isMulti")
            return is_multi

        try:
            is_multi = get_is_multi()
        except KeyError:
            if strict:
                raise
            is_multi = cast(Union[Unset, bool], UNSET)

        def get_text_value() -> Union[Unset, None, str]:
            text_value = d.pop("textValue")
            return text_value

        try:
            text_value = get_text_value()
        except KeyError:
            if strict:
                raise
            text_value = cast(Union[Unset, None, str], UNSET)

        def get_type() -> Union[Unset, FieldType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = FieldType(_type)
                except ValueError:
                    type = FieldType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, FieldType], UNSET)

        def get_value() -> Union[None, str, bool, float, FieldValue, List[str], UnknownType]:
            def _parse_value(
                data: Union[None, str, bool, float, Dict[str, Any], List[Any]]
            ) -> Union[None, str, bool, float, FieldValue, List[str], UnknownType]:
                value: Union[None, str, bool, float, FieldValue, List[str], UnknownType]
                if data is None:
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    value = FieldValue.from_dict(data, strict=True)

                    return value
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    value = cast(List[str], data)

                    return value
                except:  # noqa: E722
                    pass
                if isinstance(data, dict):
                    return UnknownType(data)
                return cast(Union[None, str, bool, float, FieldValue, List[str], UnknownType], data)

            value = _parse_value(d.pop("value"))

            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(Union[None, str, bool, float, FieldValue, List[str], UnknownType], UNSET)

        field = cls(
            display_value=display_value,
            is_multi=is_multi,
            text_value=text_value,
            type=type,
            value=value,
        )

        field.additional_properties = d
        return field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def display_value(self) -> Optional[str]:
        if isinstance(self._display_value, Unset):
            raise NotPresentError(self, "display_value")
        return self._display_value

    @display_value.setter
    def display_value(self, value: Optional[str]) -> None:
        self._display_value = value

    @display_value.deleter
    def display_value(self) -> None:
        self._display_value = UNSET

    @property
    def is_multi(self) -> bool:
        if isinstance(self._is_multi, Unset):
            raise NotPresentError(self, "is_multi")
        return self._is_multi

    @is_multi.setter
    def is_multi(self, value: bool) -> None:
        self._is_multi = value

    @is_multi.deleter
    def is_multi(self) -> None:
        self._is_multi = UNSET

    @property
    def text_value(self) -> Optional[str]:
        if isinstance(self._text_value, Unset):
            raise NotPresentError(self, "text_value")
        return self._text_value

    @text_value.setter
    def text_value(self, value: Optional[str]) -> None:
        self._text_value = value

    @text_value.deleter
    def text_value(self) -> None:
        self._text_value = UNSET

    @property
    def type(self) -> FieldType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: FieldType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def value(self) -> Optional[Union[str, bool, float, FieldValue, List[str], UnknownType]]:
        """For single link fields, use the id of the item you want to link (eg. "seq_jdf8BV24").
        For multi-link fields, use an array of ids of the items you want to link (eg. ["seq_jdf8BV24"])
        """
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: Optional[Union[str, bool, float, FieldValue, List[str], UnknownType]]) -> None:
        self._value = value



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="DnaOligosUnarchive")


@attr.s(auto_attribs=True, repr=False)
class DnaOligosUnarchive:
    """The request body for unarchiving DNA Oligos."""

    _dna_oligo_ids: List[str]

    def __repr__(self):
        fields = []
        fields.append("dna_oligo_ids={}".format(repr(self._dna_oligo_ids)))
        return "DnaOligosUnarchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        dna_oligo_ids = self._dna_oligo_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if dna_oligo_ids is not UNSET:
            field_dict["dnaOligoIds"] = dna_oligo_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_dna_oligo_ids() -> List[str]:
            dna_oligo_ids = cast(List[str], d.pop("dnaOligoIds"))

            return dna_oligo_ids

        try:
            dna_oligo_ids = get_dna_oligo_ids()
        except KeyError:
            if strict:
                raise
            dna_oligo_ids = cast(List[str], UNSET)

        dna_oligos_unarchive = cls(
            dna_oligo_ids=dna_oligo_ids,
        )

        return dna_oligos_unarchive

    @property
    def dna_oligo_ids(self) -> List[str]:
        if isinstance(self._dna_oligo_ids, Unset):
            raise NotPresentError(self, "dna_oligo_ids")
        return self._dna_oligo_ids

    @dna_oligo_ids.setter
    def dna_oligo_ids(self, value: List[str]) -> None:
        self._dna_oligo_ids = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="PlateCreateWellsAdditionalProperty")


@attr.s(auto_attribs=True, repr=False)
class PlateCreateWellsAdditionalProperty:
    """  """

    _barcode: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("barcode={}".format(repr(self._barcode)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "PlateCreateWellsAdditionalProperty({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        barcode = self._barcode

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if barcode is not UNSET:
            field_dict["barcode"] = barcode

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_barcode() -> Union[Unset, str]:
            barcode = d.pop("barcode")
            return barcode

        try:
            barcode = get_barcode()
        except KeyError:
            if strict:
                raise
            barcode = cast(Union[Unset, str], UNSET)

        plate_create_wells_additional_property = cls(
            barcode=barcode,
        )

        plate_create_wells_additional_property.additional_properties = d
        return plate_create_wells_additional_property

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def barcode(self) -> str:
        if isinstance(self._barcode, Unset):
            raise NotPresentError(self, "barcode")
        return self._barcode

    @barcode.setter
    def barcode(self, value: str) -> None:
        self._barcode = value

    @barcode.deleter
    def barcode(self) -> None:
        self._barcode = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.canvas_initialize_webhook import CanvasInitializeWebhook
from ..models.canvas_interaction_webhook import CanvasInteractionWebhook
from ..models.webhook_envelope_app import WebhookEnvelopeApp
from ..models.webhook_envelope_channel import WebhookEnvelopeChannel
from ..models.webhook_envelope_version import WebhookEnvelopeVersion
from ..types import UNSET, Unset

T = TypeVar("T", bound="WebhookEnvelope")


@attr.s(auto_attribs=True, repr=False)
class WebhookEnvelope:
    """  """

    _app: WebhookEnvelopeApp
    _base_url: str
    _channel: WebhookEnvelopeChannel
    _message: Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]
    _tenant_id: str
    _version: WebhookEnvelopeVersion
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("app={}".format(repr(self._app)))
        fields.append("base_url={}".format(repr(self._base_url)))
        fields.append("channel={}".format(repr(self._channel)))
        fields.append("message={}".format(repr(self._message)))
        fields.append("tenant_id={}".format(repr(self._tenant_id)))
        fields.append("version={}".format(repr(self._version)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WebhookEnvelope({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        app = self._app.to_dict()

        base_url = self._base_url
        channel = self._channel.value

        if isinstance(self._message, UnknownType):
            message = self._message.value
        elif isinstance(self._message, CanvasInteractionWebhook):
            message = self._message.to_dict()

        else:
            message = self._message.to_dict()

        tenant_id = self._tenant_id
        version = self._version.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if app is not UNSET:
            field_dict["app"] = app
        if base_url is not UNSET:
            field_dict["baseURL"] = base_url
        if channel is not UNSET:
            field_dict["channel"] = channel
        if message is not UNSET:
            field_dict["message"] = message
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_app() -> WebhookEnvelopeApp:
            app = WebhookEnvelopeApp.from_dict(d.pop("app"), strict=False)

            return app

        try:
            app = get_app()
        except KeyError:
            if strict:
                raise
            app = cast(WebhookEnvelopeApp, UNSET)

        def get_base_url() -> str:
            base_url = d.pop("baseURL")
            return base_url

        try:
            base_url = get_base_url()
        except KeyError:
            if strict:
                raise
            base_url = cast(str, UNSET)

        def get_channel() -> WebhookEnvelopeChannel:
            _channel = d.pop("channel")
            try:
                channel = WebhookEnvelopeChannel(_channel)
            except ValueError:
                channel = WebhookEnvelopeChannel.of_unknown(_channel)

            return channel

        try:
            channel = get_channel()
        except KeyError:
            if strict:
                raise
            channel = cast(WebhookEnvelopeChannel, UNSET)

        def get_message() -> Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]:
            message: Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]
            _message = d.pop("message")

            if True:
                discriminator = _message["type"]
                if discriminator == "v0-beta.canvas.initialized":
                    message = CanvasInitializeWebhook.from_dict(_message)
                elif discriminator == "v0-beta.canvas.userInteracted":
                    message = CanvasInteractionWebhook.from_dict(_message)
                else:
                    message = UnknownType(value=_message)

            return message

        try:
            message = get_message()
        except KeyError:
            if strict:
                raise
            message = cast(Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType], UNSET)

        def get_tenant_id() -> str:
            tenant_id = d.pop("tenantId")
            return tenant_id

        try:
            tenant_id = get_tenant_id()
        except KeyError:
            if strict:
                raise
            tenant_id = cast(str, UNSET)

        def get_version() -> WebhookEnvelopeVersion:
            _version = d.pop("version")
            try:
                version = WebhookEnvelopeVersion(_version)
            except ValueError:
                version = WebhookEnvelopeVersion.of_unknown(_version)

            return version

        try:
            version = get_version()
        except KeyError:
            if strict:
                raise
            version = cast(WebhookEnvelopeVersion, UNSET)

        webhook_envelope = cls(
            app=app,
            base_url=base_url,
            channel=channel,
            message=message,
            tenant_id=tenant_id,
            version=version,
        )

        webhook_envelope.additional_properties = d
        return webhook_envelope

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def app(self) -> WebhookEnvelopeApp:
        if isinstance(self._app, Unset):
            raise NotPresentError(self, "app")
        return self._app

    @app.setter
    def app(self, value: WebhookEnvelopeApp) -> None:
        self._app = value

    @property
    def base_url(self) -> str:
        """ Base tenant URL from which the webhook is coming """
        if isinstance(self._base_url, Unset):
            raise NotPresentError(self, "base_url")
        return self._base_url

    @base_url.setter
    def base_url(self, value: str) -> None:
        self._base_url = value

    @property
    def channel(self) -> WebhookEnvelopeChannel:
        if isinstance(self._channel, Unset):
            raise NotPresentError(self, "channel")
        return self._channel

    @channel.setter
    def channel(self, value: WebhookEnvelopeChannel) -> None:
        self._channel = value

    @property
    def message(self) -> Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]:
        if isinstance(self._message, Unset):
            raise NotPresentError(self, "message")
        return self._message

    @message.setter
    def message(self, value: Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]) -> None:
        self._message = value

    @property
    def tenant_id(self) -> str:
        """ Global tenant id from which webhook is coming """
        if isinstance(self._tenant_id, Unset):
            raise NotPresentError(self, "tenant_id")
        return self._tenant_id

    @tenant_id.setter
    def tenant_id(self, value: str) -> None:
        self._tenant_id = value

    @property
    def version(self) -> WebhookEnvelopeVersion:
        """ Version of the webhook envelope shape. Always 0 for now. """
        if isinstance(self._version, Unset):
            raise NotPresentError(self, "version")
        return self._version

    @version.setter
    def version(self, value: WebhookEnvelopeVersion) -> None:
        self._version = value



from typing import Any, cast, Dict, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobPartCreate")


@attr.s(auto_attribs=True, repr=False)
class BlobPartCreate:
    """  """

    _data64: str
    _md5: str
    _part_number: int

    def __repr__(self):
        fields = []
        fields.append("data64={}".format(repr(self._data64)))
        fields.append("md5={}".format(repr(self._md5)))
        fields.append("part_number={}".format(repr(self._part_number)))
        return "BlobPartCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        data64 = self._data64
        md5 = self._md5
        part_number = self._part_number

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if data64 is not UNSET:
            field_dict["data64"] = data64
        if md5 is not UNSET:
            field_dict["md5"] = md5
        if part_number is not UNSET:
            field_dict["partNumber"] = part_number

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_data64() -> str:
            data64 = d.pop("data64")
            return data64

        try:
            data64 = get_data64()
        except KeyError:
            if strict:
                raise
            data64 = cast(str, UNSET)

        def get_md5() -> str:
            md5 = d.pop("md5")
            return md5

        try:
            md5 = get_md5()
        except KeyError:
            if strict:
                raise
            md5 = cast(str, UNSET)

        def get_part_number() -> int:
            part_number = d.pop("partNumber")
            return part_number

        try:
            part_number = get_part_number()
        except KeyError:
            if strict:
                raise
            part_number = cast(int, UNSET)

        blob_part_create = cls(
            data64=data64,
            md5=md5,
            part_number=part_number,
        )

        return blob_part_create

    @property
    def data64(self) -> str:
        if isinstance(self._data64, Unset):
            raise NotPresentError(self, "data64")
        return self._data64

    @data64.setter
    def data64(self, value: str) -> None:
        self._data64 = value

    @property
    def md5(self) -> str:
        if isinstance(self._md5, Unset):
            raise NotPresentError(self, "md5")
        return self._md5

    @md5.setter
    def md5(self, value: str) -> None:
        self._md5 = value

    @property
    def part_number(self) -> int:
        """An integer between 1 to 10,000, inclusive. The part number must be unique per part and indicates the ordering of the part inside the final blob. The part numbers do not need to be consecutive."""
        if isinstance(self._part_number, Unset):
            raise NotPresentError(self, "part_number")
        return self._part_number

    @part_number.setter
    def part_number(self, value: int) -> None:
        self._part_number = value



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserActivity")


@attr.s(auto_attribs=True, repr=False)
class UserActivity:
    """  """

    _last_seen: Union[Unset, None, datetime.datetime] = UNSET
    _user_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("last_seen={}".format(repr(self._last_seen)))
        fields.append("user_id={}".format(repr(self._user_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "UserActivity({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        last_seen: Union[Unset, None, str] = UNSET
        if not isinstance(self._last_seen, Unset):
            last_seen = self._last_seen.isoformat() if self._last_seen else None

        user_id = self._user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if last_seen is not UNSET:
            field_dict["lastSeen"] = last_seen
        if user_id is not UNSET:
            field_dict["userId"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_last_seen() -> Union[Unset, None, datetime.datetime]:
            last_seen: Union[Unset, None, datetime.datetime] = UNSET
            _last_seen = d.pop("lastSeen")
            if _last_seen is not None and not isinstance(_last_seen, Unset):
                last_seen = isoparse(cast(str, _last_seen))

            return last_seen

        try:
            last_seen = get_last_seen()
        except KeyError:
            if strict:
                raise
            last_seen = cast(Union[Unset, None, datetime.datetime], UNSET)

        def get_user_id() -> Union[Unset, str]:
            user_id = d.pop("userId")
            return user_id

        try:
            user_id = get_user_id()
        except KeyError:
            if strict:
                raise
            user_id = cast(Union[Unset, str], UNSET)

        user_activity = cls(
            last_seen=last_seen,
            user_id=user_id,
        )

        user_activity.additional_properties = d
        return user_activity

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def last_seen(self) -> Optional[datetime.datetime]:
        if isinstance(self._last_seen, Unset):
            raise NotPresentError(self, "last_seen")
        return self._last_seen

    @last_seen.setter
    def last_seen(self, value: Optional[datetime.datetime]) -> None:
        self._last_seen = value

    @last_seen.deleter
    def last_seen(self) -> None:
        self._last_seen = UNSET

    @property
    def user_id(self) -> str:
        if isinstance(self._user_id, Unset):
            raise NotPresentError(self, "user_id")
        return self._user_id

    @user_id.setter
    def user_id(self, value: str) -> None:
        self._user_id = value

    @user_id.deleter
    def user_id(self) -> None:
        self._user_id = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError, UnknownType
from ..models.organization import Organization
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeatureLibrary")


@attr.s(auto_attribs=True, repr=False)
class FeatureLibrary:
    """ A feature library """

    _created_at: Union[Unset, datetime.datetime] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _owner: Union[Unset, Organization, UserSummary, UnknownType] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _description: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("owner={}".format(repr(self._owner)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "FeatureLibrary({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        owner: Union[Unset, Dict[str, Any]]
        if isinstance(self._owner, Unset):
            owner = UNSET
        elif isinstance(self._owner, UnknownType):
            owner = self._owner.value
        elif isinstance(self._owner, Organization):
            owner = UNSET
            if not isinstance(self._owner, Unset):
                owner = self._owner.to_dict()

        else:
            owner = UNSET
            if not isinstance(self._owner, Unset):
                owner = self._owner.to_dict()

        web_url = self._web_url
        description = self._description
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if owner is not UNSET:
            field_dict["owner"] = owner
        if web_url is not UNSET:
            field_dict["webURL"] = web_url
        if description is not UNSET:
            field_dict["description"] = description
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_owner() -> Union[Unset, Organization, UserSummary, UnknownType]:
            def _parse_owner(
                data: Union[Unset, Dict[str, Any]]
            ) -> Union[Unset, Organization, UserSummary, UnknownType]:
                owner: Union[Unset, Organization, UserSummary, UnknownType]
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    owner = UNSET
                    _owner = data

                    if not isinstance(_owner, Unset):
                        owner = Organization.from_dict(_owner)

                    return owner
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    owner = UNSET
                    _owner = data

                    if not isinstance(_owner, Unset):
                        owner = UserSummary.from_dict(_owner)

                    return owner
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            owner = _parse_owner(d.pop("owner"))

            return owner

        try:
            owner = get_owner()
        except KeyError:
            if strict:
                raise
            owner = cast(Union[Unset, Organization, UserSummary, UnknownType], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        feature_library = cls(
            created_at=created_at,
            id=id,
            modified_at=modified_at,
            owner=owner,
            web_url=web_url,
            description=description,
            name=name,
        )

        feature_library.additional_properties = d
        return feature_library

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime the Feature Library was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def id(self) -> str:
        """ The id of the feature library """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the Feature Library was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def owner(self) -> Union[Organization, UserSummary, UnknownType]:
        if isinstance(self._owner, Unset):
            raise NotPresentError(self, "owner")
        return self._owner

    @owner.setter
    def owner(self, value: Union[Organization, UserSummary, UnknownType]) -> None:
        self._owner = value

    @owner.deleter
    def owner(self) -> None:
        self._owner = UNSET

    @property
    def web_url(self) -> str:
        """ The Benchling web UI url to view the Feature Library """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def description(self) -> str:
        """ The description for the feature library """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def name(self) -> str:
        """ The name of the feature library """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="InaccessibleResource")


@attr.s(auto_attribs=True, repr=False)
class InaccessibleResource:
    """  """

    _inaccessible_id: Union[Unset, str] = UNSET
    _type: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("inaccessible_id={}".format(repr(self._inaccessible_id)))
        fields.append("type={}".format(repr(self._type)))
        return "InaccessibleResource({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        inaccessible_id = self._inaccessible_id
        type = self._type

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if inaccessible_id is not UNSET:
            field_dict["inaccessibleId"] = inaccessible_id
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_inaccessible_id() -> Union[Unset, str]:
            inaccessible_id = d.pop("inaccessibleId")
            return inaccessible_id

        try:
            inaccessible_id = get_inaccessible_id()
        except KeyError:
            if strict:
                raise
            inaccessible_id = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        inaccessible_resource = cls(
            inaccessible_id=inaccessible_id,
            type=type,
        )

        return inaccessible_resource

    @property
    def inaccessible_id(self) -> str:
        if isinstance(self._inaccessible_id, Unset):
            raise NotPresentError(self, "inaccessible_id")
        return self._inaccessible_id

    @inaccessible_id.setter
    def inaccessible_id(self, value: str) -> None:
        self._inaccessible_id = value

    @inaccessible_id.deleter
    def inaccessible_id(self) -> None:
        self._inaccessible_id = UNSET

    @property
    def type(self) -> str:
        """The type of this inaccessible item. Example values: "custom_entity", "container", "plate", "dna_sequence" """
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.array_element_app_config_item import ArrayElementAppConfigItem
from ..models.boolean_app_config_item import BooleanAppConfigItem
from ..models.date_app_config_item import DateAppConfigItem
from ..models.datetime_app_config_item import DatetimeAppConfigItem
from ..models.entity_schema_app_config_item import EntitySchemaAppConfigItem
from ..models.field_app_config_item import FieldAppConfigItem
from ..models.float_app_config_item import FloatAppConfigItem
from ..models.generic_api_identified_app_config_item import GenericApiIdentifiedAppConfigItem
from ..models.integer_app_config_item import IntegerAppConfigItem
from ..models.json_app_config_item import JsonAppConfigItem
from ..models.secure_text_app_config_item import SecureTextAppConfigItem
from ..models.text_app_config_item import TextAppConfigItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppConfigurationPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class AppConfigurationPaginatedList:
    """  """

    _app_configuration_items: Union[
        Unset,
        List[
            Union[
                ArrayElementAppConfigItem,
                DateAppConfigItem,
                DatetimeAppConfigItem,
                JsonAppConfigItem,
                EntitySchemaAppConfigItem,
                FieldAppConfigItem,
                BooleanAppConfigItem,
                IntegerAppConfigItem,
                FloatAppConfigItem,
                TextAppConfigItem,
                GenericApiIdentifiedAppConfigItem,
                SecureTextAppConfigItem,
                UnknownType,
            ]
        ],
    ] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("app_configuration_items={}".format(repr(self._app_configuration_items)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "AppConfigurationPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        app_configuration_items: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._app_configuration_items, Unset):
            app_configuration_items = []
            for app_configuration_items_item_data in self._app_configuration_items:
                if isinstance(app_configuration_items_item_data, UnknownType):
                    app_configuration_items_item = app_configuration_items_item_data.value
                elif isinstance(app_configuration_items_item_data, ArrayElementAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, DateAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, DatetimeAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, JsonAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, EntitySchemaAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, FieldAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, BooleanAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, IntegerAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, FloatAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, TextAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                elif isinstance(app_configuration_items_item_data, GenericApiIdentifiedAppConfigItem):
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                else:
                    app_configuration_items_item = app_configuration_items_item_data.to_dict()

                app_configuration_items.append(app_configuration_items_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if app_configuration_items is not UNSET:
            field_dict["appConfigurationItems"] = app_configuration_items
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_app_configuration_items() -> Union[
            Unset,
            List[
                Union[
                    ArrayElementAppConfigItem,
                    DateAppConfigItem,
                    DatetimeAppConfigItem,
                    JsonAppConfigItem,
                    EntitySchemaAppConfigItem,
                    FieldAppConfigItem,
                    BooleanAppConfigItem,
                    IntegerAppConfigItem,
                    FloatAppConfigItem,
                    TextAppConfigItem,
                    GenericApiIdentifiedAppConfigItem,
                    SecureTextAppConfigItem,
                    UnknownType,
                ]
            ],
        ]:
            app_configuration_items = []
            _app_configuration_items = d.pop("appConfigurationItems")
            for app_configuration_items_item_data in _app_configuration_items or []:

                def _parse_app_configuration_items_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    ArrayElementAppConfigItem,
                    DateAppConfigItem,
                    DatetimeAppConfigItem,
                    JsonAppConfigItem,
                    EntitySchemaAppConfigItem,
                    FieldAppConfigItem,
                    BooleanAppConfigItem,
                    IntegerAppConfigItem,
                    FloatAppConfigItem,
                    TextAppConfigItem,
                    GenericApiIdentifiedAppConfigItem,
                    SecureTextAppConfigItem,
                    UnknownType,
                ]:
                    app_configuration_items_item: Union[
                        ArrayElementAppConfigItem,
                        DateAppConfigItem,
                        DatetimeAppConfigItem,
                        JsonAppConfigItem,
                        EntitySchemaAppConfigItem,
                        FieldAppConfigItem,
                        BooleanAppConfigItem,
                        IntegerAppConfigItem,
                        FloatAppConfigItem,
                        TextAppConfigItem,
                        GenericApiIdentifiedAppConfigItem,
                        SecureTextAppConfigItem,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("type"))
                    if discriminator_value is not None:
                        app_config_item: Union[
                            ArrayElementAppConfigItem,
                            DateAppConfigItem,
                            DatetimeAppConfigItem,
                            JsonAppConfigItem,
                            EntitySchemaAppConfigItem,
                            FieldAppConfigItem,
                            BooleanAppConfigItem,
                            IntegerAppConfigItem,
                            FloatAppConfigItem,
                            TextAppConfigItem,
                            GenericApiIdentifiedAppConfigItem,
                            SecureTextAppConfigItem,
                            UnknownType,
                        ]
                        if discriminator_value == "aa_sequence":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "array_element":
                            app_config_item = ArrayElementAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "boolean":
                            app_config_item = BooleanAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "box":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "box_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "container":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "container_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "custom_entity":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "date":
                            app_config_item = DateAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "datetime":
                            app_config_item = DatetimeAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "dna_oligo":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "dna_sequence":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "dropdown":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "dropdown_option":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "entity_schema":
                            app_config_item = EntitySchemaAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "entry":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "entry_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "field":
                            app_config_item = FieldAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "float":
                            app_config_item = FloatAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "folder":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "integer":
                            app_config_item = IntegerAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "json":
                            app_config_item = JsonAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "location":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "location_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "mixture":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "molecule":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "plate":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "plate_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "project":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "registry":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "request_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "result_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "rna_oligo":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "rna_sequence":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "run_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "secure_text":
                            app_config_item = SecureTextAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "text":
                            app_config_item = TextAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "workflow_task_schema":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "workflow_task_status":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item
                        if discriminator_value == "worklist":
                            app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=False)

                            return app_config_item

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = ArrayElementAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = DateAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = DatetimeAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = JsonAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = EntitySchemaAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = FieldAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = BooleanAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = IntegerAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = FloatAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = TextAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = GenericApiIdentifiedAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item = SecureTextAppConfigItem.from_dict(data, strict=True)

                        return app_config_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                app_configuration_items_item = _parse_app_configuration_items_item(
                    app_configuration_items_item_data
                )

                app_configuration_items.append(app_configuration_items_item)

            return app_configuration_items

        try:
            app_configuration_items = get_app_configuration_items()
        except KeyError:
            if strict:
                raise
            app_configuration_items = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            ArrayElementAppConfigItem,
                            DateAppConfigItem,
                            DatetimeAppConfigItem,
                            JsonAppConfigItem,
                            EntitySchemaAppConfigItem,
                            FieldAppConfigItem,
                            BooleanAppConfigItem,
                            IntegerAppConfigItem,
                            FloatAppConfigItem,
                            TextAppConfigItem,
                            GenericApiIdentifiedAppConfigItem,
                            SecureTextAppConfigItem,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        app_configuration_paginated_list = cls(
            app_configuration_items=app_configuration_items,
            next_token=next_token,
        )

        return app_configuration_paginated_list

    @property
    def app_configuration_items(
        self,
    ) -> List[
        Union[
            ArrayElementAppConfigItem,
            DateAppConfigItem,
            DatetimeAppConfigItem,
            JsonAppConfigItem,
            EntitySchemaAppConfigItem,
            FieldAppConfigItem,
            BooleanAppConfigItem,
            IntegerAppConfigItem,
            FloatAppConfigItem,
            TextAppConfigItem,
            GenericApiIdentifiedAppConfigItem,
            SecureTextAppConfigItem,
            UnknownType,
        ]
    ]:
        if isinstance(self._app_configuration_items, Unset):
            raise NotPresentError(self, "app_configuration_items")
        return self._app_configuration_items

    @app_configuration_items.setter
    def app_configuration_items(
        self,
        value: List[
            Union[
                ArrayElementAppConfigItem,
                DateAppConfigItem,
                DatetimeAppConfigItem,
                JsonAppConfigItem,
                EntitySchemaAppConfigItem,
                FieldAppConfigItem,
                BooleanAppConfigItem,
                IntegerAppConfigItem,
                FloatAppConfigItem,
                TextAppConfigItem,
                GenericApiIdentifiedAppConfigItem,
                SecureTextAppConfigItem,
                UnknownType,
            ]
        ],
    ) -> None:
        self._app_configuration_items = value

    @app_configuration_items.deleter
    def app_configuration_items(self) -> None:
        self._app_configuration_items = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.inaccessible_resource import InaccessibleResource
from ..models.linked_app_config_resource_summary import LinkedAppConfigResourceSummary

LinkedAppConfigResource = Union[LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="OptimizeCodonsHairpinParameters")


@attr.s(auto_attribs=True, repr=False)
class OptimizeCodonsHairpinParameters:
    """These parameters are applied in the AvoidHairpins specification in DNAChisel. If hairpinParameters is not specified, hairpins will not be avoided."""

    _stem: Union[Unset, int] = 20
    _window: Union[Unset, int] = 200

    def __repr__(self):
        fields = []
        fields.append("stem={}".format(repr(self._stem)))
        fields.append("window={}".format(repr(self._window)))
        return "OptimizeCodonsHairpinParameters({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        stem = self._stem
        window = self._window

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if stem is not UNSET:
            field_dict["stem"] = stem
        if window is not UNSET:
            field_dict["window"] = window

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_stem() -> Union[Unset, int]:
            stem = d.pop("stem")
            return stem

        try:
            stem = get_stem()
        except KeyError:
            if strict:
                raise
            stem = cast(Union[Unset, int], UNSET)

        def get_window() -> Union[Unset, int]:
            window = d.pop("window")
            return window

        try:
            window = get_window()
        except KeyError:
            if strict:
                raise
            window = cast(Union[Unset, int], UNSET)

        optimize_codons_hairpin_parameters = cls(
            stem=stem,
            window=window,
        )

        return optimize_codons_hairpin_parameters

    @property
    def stem(self) -> int:
        if isinstance(self._stem, Unset):
            raise NotPresentError(self, "stem")
        return self._stem

    @stem.setter
    def stem(self, value: int) -> None:
        self._stem = value

    @stem.deleter
    def stem(self) -> None:
        self._stem = UNSET

    @property
    def window(self) -> int:
        if isinstance(self._window, Unset):
            raise NotPresentError(self, "window")
        return self._window

    @window.setter
    def window(self, value: int) -> None:
        self._window = value

    @window.deleter
    def window(self) -> None:
        self._window = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowRouterFunction")


@attr.s(auto_attribs=True, repr=False)
class WorkflowRouterFunction:
    """  """

    _edge_config_id: Union[Unset, str] = UNSET
    _expression: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _is_default: Union[Unset, None] = UNSET

    def __repr__(self):
        fields = []
        fields.append("edge_config_id={}".format(repr(self._edge_config_id)))
        fields.append("expression={}".format(repr(self._expression)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("is_default={}".format(repr(self._is_default)))
        return "WorkflowRouterFunction({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        edge_config_id = self._edge_config_id
        expression = self._expression
        id = self._id
        is_default = None

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if edge_config_id is not UNSET:
            field_dict["edgeConfigId"] = edge_config_id
        if expression is not UNSET:
            field_dict["expression"] = expression
        if id is not UNSET:
            field_dict["id"] = id
        if is_default is not UNSET:
            field_dict["isDefault"] = is_default

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_edge_config_id() -> Union[Unset, str]:
            edge_config_id = d.pop("edgeConfigId")
            return edge_config_id

        try:
            edge_config_id = get_edge_config_id()
        except KeyError:
            if strict:
                raise
            edge_config_id = cast(Union[Unset, str], UNSET)

        def get_expression() -> Union[Unset, str]:
            expression = d.pop("expression")
            return expression

        try:
            expression = get_expression()
        except KeyError:
            if strict:
                raise
            expression = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_is_default() -> Union[Unset, None]:
            is_default = None

            return is_default

        try:
            is_default = get_is_default()
        except KeyError:
            if strict:
                raise
            is_default = cast(Union[Unset, None], UNSET)

        workflow_router_function = cls(
            edge_config_id=edge_config_id,
            expression=expression,
            id=id,
            is_default=is_default,
        )

        return workflow_router_function

    @property
    def edge_config_id(self) -> str:
        """ The ID of the workflow flowchart edge config associated with this function """
        if isinstance(self._edge_config_id, Unset):
            raise NotPresentError(self, "edge_config_id")
        return self._edge_config_id

    @edge_config_id.setter
    def edge_config_id(self, value: str) -> None:
        self._edge_config_id = value

    @edge_config_id.deleter
    def edge_config_id(self) -> None:
        self._edge_config_id = UNSET

    @property
    def expression(self) -> str:
        """ A JSON object representing the expression associated with this function """
        if isinstance(self._expression, Unset):
            raise NotPresentError(self, "expression")
        return self._expression

    @expression.setter
    def expression(self, value: str) -> None:
        self._expression = value

    @expression.deleter
    def expression(self) -> None:
        self._expression = UNSET

    @property
    def id(self) -> str:
        """ The function associated with a router node """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def is_default(self) -> None:
        if isinstance(self._is_default, Unset):
            raise NotPresentError(self, "is_default")
        return self._is_default

    @is_default.setter
    def is_default(self, value: None) -> None:
        self._is_default = value

    @is_default.deleter
    def is_default(self) -> None:
        self._is_default = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entity_search_form_instance_provider_type import EntitySearchFormInstanceProviderType
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntitySearchFormInstanceProvider")


@attr.s(auto_attribs=True, repr=False)
class EntitySearchFormInstanceProvider:
    """  """

    _target_field_key: Union[Unset, str] = UNSET
    _type: Union[Unset, EntitySearchFormInstanceProviderType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("target_field_key={}".format(repr(self._target_field_key)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntitySearchFormInstanceProvider({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        target_field_key = self._target_field_key
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if target_field_key is not UNSET:
            field_dict["targetFieldKey"] = target_field_key
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_target_field_key() -> Union[Unset, str]:
            target_field_key = d.pop("targetFieldKey")
            return target_field_key

        try:
            target_field_key = get_target_field_key()
        except KeyError:
            if strict:
                raise
            target_field_key = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, EntitySearchFormInstanceProviderType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = EntitySearchFormInstanceProviderType(_type)
                except ValueError:
                    type = EntitySearchFormInstanceProviderType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, EntitySearchFormInstanceProviderType], UNSET)

        entity_search_form_instance_provider = cls(
            target_field_key=target_field_key,
            type=type,
        )

        entity_search_form_instance_provider.additional_properties = d
        return entity_search_form_instance_provider

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def target_field_key(self) -> str:
        """Where the entities that we search for should be populated within our form. Note here that we inherit any considerations and limitations on the values these entities can take on, like schemaId, from the definition of the field we're routing them to."""
        if isinstance(self._target_field_key, Unset):
            raise NotPresentError(self, "target_field_key")
        return self._target_field_key

    @target_field_key.setter
    def target_field_key(self, value: str) -> None:
        self._target_field_key = value

    @target_field_key.deleter
    def target_field_key(self) -> None:
        self._target_field_key = UNSET

    @property
    def type(self) -> EntitySearchFormInstanceProviderType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: EntitySearchFormInstanceProviderType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeatureLibraryCreate")


@attr.s(auto_attribs=True, repr=False)
class FeatureLibraryCreate:
    """ Inputs for creating a feature library """

    _organization_id: Union[Unset, str] = UNSET
    _description: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("organization_id={}".format(repr(self._organization_id)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("name={}".format(repr(self._name)))
        return "FeatureLibraryCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        organization_id = self._organization_id
        description = self._description
        name = self._name

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if description is not UNSET:
            field_dict["description"] = description
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_organization_id() -> Union[Unset, str]:
            organization_id = d.pop("organizationId")
            return organization_id

        try:
            organization_id = get_organization_id()
        except KeyError:
            if strict:
                raise
            organization_id = cast(Union[Unset, str], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        feature_library_create = cls(
            organization_id=organization_id,
            description=description,
            name=name,
        )

        return feature_library_create

    @property
    def organization_id(self) -> str:
        """The organization that will own the feature library. The requesting user must be an administrator of the organization. If unspecified and the organization allows personal ownables, then the requesting user will own the feature library"""
        if isinstance(self._organization_id, Unset):
            raise NotPresentError(self, "organization_id")
        return self._organization_id

    @organization_id.setter
    def organization_id(self, value: str) -> None:
        self._organization_id = value

    @organization_id.deleter
    def organization_id(self) -> None:
        self._organization_id = UNSET

    @property
    def description(self) -> str:
        """ The description for the feature library """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def name(self) -> str:
        """ The name of the feature library """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.blob_part import BlobPart
from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobComplete")


@attr.s(auto_attribs=True, repr=False)
class BlobComplete:
    """  """

    _parts: Union[Unset, List[BlobPart]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("parts={}".format(repr(self._parts)))
        return "BlobComplete({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        parts: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._parts, Unset):
            parts = []
            for parts_item_data in self._parts:
                parts_item = parts_item_data.to_dict()

                parts.append(parts_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if parts is not UNSET:
            field_dict["parts"] = parts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_parts() -> Union[Unset, List[BlobPart]]:
            parts = []
            _parts = d.pop("parts")
            for parts_item_data in _parts or []:
                parts_item = BlobPart.from_dict(parts_item_data, strict=False)

                parts.append(parts_item)

            return parts

        try:
            parts = get_parts()
        except KeyError:
            if strict:
                raise
            parts = cast(Union[Unset, List[BlobPart]], UNSET)

        blob_complete = cls(
            parts=parts,
        )

        return blob_complete

    @property
    def parts(self) -> List[BlobPart]:
        if isinstance(self._parts, Unset):
            raise NotPresentError(self, "parts")
        return self._parts

    @parts.setter
    def parts(self, value: List[BlobPart]) -> None:
        self._parts = value

    @parts.deleter
    def parts(self) -> None:
        self._parts = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationOutputProcessorArchivalChange")


@attr.s(auto_attribs=True, repr=False)
class AutomationOutputProcessorArchivalChange:
    """ IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of any linked Results that were archived / unarchived. """

    _automation_output_processor_ids: Union[Unset, List[str]] = UNSET
    _result_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append(
            "automation_output_processor_ids={}".format(repr(self._automation_output_processor_ids))
        )
        fields.append("result_ids={}".format(repr(self._result_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AutomationOutputProcessorArchivalChange({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        automation_output_processor_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._automation_output_processor_ids, Unset):
            automation_output_processor_ids = self._automation_output_processor_ids

        result_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._result_ids, Unset):
            result_ids = self._result_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if automation_output_processor_ids is not UNSET:
            field_dict["automationOutputProcessorIds"] = automation_output_processor_ids
        if result_ids is not UNSET:
            field_dict["resultIds"] = result_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_automation_output_processor_ids() -> Union[Unset, List[str]]:
            automation_output_processor_ids = cast(List[str], d.pop("automationOutputProcessorIds"))

            return automation_output_processor_ids

        try:
            automation_output_processor_ids = get_automation_output_processor_ids()
        except KeyError:
            if strict:
                raise
            automation_output_processor_ids = cast(Union[Unset, List[str]], UNSET)

        def get_result_ids() -> Union[Unset, List[str]]:
            result_ids = cast(List[str], d.pop("resultIds"))

            return result_ids

        try:
            result_ids = get_result_ids()
        except KeyError:
            if strict:
                raise
            result_ids = cast(Union[Unset, List[str]], UNSET)

        automation_output_processor_archival_change = cls(
            automation_output_processor_ids=automation_output_processor_ids,
            result_ids=result_ids,
        )

        automation_output_processor_archival_change.additional_properties = d
        return automation_output_processor_archival_change

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def automation_output_processor_ids(self) -> List[str]:
        if isinstance(self._automation_output_processor_ids, Unset):
            raise NotPresentError(self, "automation_output_processor_ids")
        return self._automation_output_processor_ids

    @automation_output_processor_ids.setter
    def automation_output_processor_ids(self, value: List[str]) -> None:
        self._automation_output_processor_ids = value

    @automation_output_processor_ids.deleter
    def automation_output_processor_ids(self) -> None:
        self._automation_output_processor_ids = UNSET

    @property
    def result_ids(self) -> List[str]:
        if isinstance(self._result_ids, Unset):
            raise NotPresentError(self, "result_ids")
        return self._result_ids

    @result_ids.setter
    def result_ids(self, value: List[str]) -> None:
        self._result_ids = value

    @result_ids.deleter
    def result_ids(self) -> None:
        self._result_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.org_membership_collaborator_role import OrgMembershipCollaboratorRole
from ..models.org_membership_collaborator_type import OrgMembershipCollaboratorType
from ..models.organization_summary import OrganizationSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="OrgMembershipCollaborator")


@attr.s(auto_attribs=True, repr=False)
class OrgMembershipCollaborator:
    """  """

    _organization: Union[Unset, OrganizationSummary] = UNSET
    _role: Union[Unset, OrgMembershipCollaboratorRole] = UNSET
    _type: Union[Unset, OrgMembershipCollaboratorType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("organization={}".format(repr(self._organization)))
        fields.append("role={}".format(repr(self._role)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "OrgMembershipCollaborator({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        organization: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._organization, Unset):
            organization = self._organization.to_dict()

        role: Union[Unset, int] = UNSET
        if not isinstance(self._role, Unset):
            role = self._role.value

        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if organization is not UNSET:
            field_dict["organization"] = organization
        if role is not UNSET:
            field_dict["role"] = role
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_organization() -> Union[Unset, OrganizationSummary]:
            organization: Union[Unset, Union[Unset, OrganizationSummary]] = UNSET
            _organization = d.pop("organization")

            if not isinstance(_organization, Unset):
                organization = OrganizationSummary.from_dict(_organization)

            return organization

        try:
            organization = get_organization()
        except KeyError:
            if strict:
                raise
            organization = cast(Union[Unset, OrganizationSummary], UNSET)

        def get_role() -> Union[Unset, OrgMembershipCollaboratorRole]:
            role = UNSET
            _role = d.pop("role")
            if _role is not None and _role is not UNSET:
                try:
                    role = OrgMembershipCollaboratorRole(_role)
                except ValueError:
                    role = OrgMembershipCollaboratorRole.of_unknown(_role)

            return role

        try:
            role = get_role()
        except KeyError:
            if strict:
                raise
            role = cast(Union[Unset, OrgMembershipCollaboratorRole], UNSET)

        def get_type() -> Union[Unset, OrgMembershipCollaboratorType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = OrgMembershipCollaboratorType(_type)
                except ValueError:
                    type = OrgMembershipCollaboratorType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, OrgMembershipCollaboratorType], UNSET)

        org_membership_collaborator = cls(
            organization=organization,
            role=role,
            type=type,
        )

        org_membership_collaborator.additional_properties = d
        return org_membership_collaborator

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def organization(self) -> OrganizationSummary:
        if isinstance(self._organization, Unset):
            raise NotPresentError(self, "organization")
        return self._organization

    @organization.setter
    def organization(self, value: OrganizationSummary) -> None:
        self._organization = value

    @organization.deleter
    def organization(self) -> None:
        self._organization = UNSET

    @property
    def role(self) -> OrgMembershipCollaboratorRole:
        if isinstance(self._role, Unset):
            raise NotPresentError(self, "role")
        return self._role

    @role.setter
    def role(self, value: OrgMembershipCollaboratorRole) -> None:
        self._role = value

    @role.deleter
    def role(self) -> None:
        self._role = UNSET

    @property
    def type(self) -> OrgMembershipCollaboratorType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: OrgMembershipCollaboratorType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.creation_origin import CreationOrigin
from ..models.fields import Fields
from ..models.user_summary import UserSummary
from ..models.workflow_output_summary import WorkflowOutputSummary
from ..models.workflow_task_execution_origin import WorkflowTaskExecutionOrigin
from ..models.workflow_task_execution_type import WorkflowTaskExecutionType
from ..models.workflow_task_group_summary import WorkflowTaskGroupSummary
from ..models.workflow_task_status import WorkflowTaskStatus
from ..models.workflow_task_summary import WorkflowTaskSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTask")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTask:
    """  """

    _execution_type: Union[Unset, WorkflowTaskExecutionType] = UNSET
    _assignee: Union[Unset, None, UserSummary] = UNSET
    _cloned_from: Union[Unset, None, WorkflowTaskSummary] = UNSET
    _creation_origin: Union[Unset, CreationOrigin] = UNSET
    _creator: Union[Unset, UserSummary] = UNSET
    _execution_origin: Union[Unset, None, WorkflowTaskExecutionOrigin] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _outputs: Union[Unset, List[WorkflowOutputSummary]] = UNSET
    _scheduled_on: Union[Unset, None, datetime.date] = UNSET
    _status: Union[Unset, WorkflowTaskStatus] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _workflow_task_group: Union[Unset, WorkflowTaskGroupSummary] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("execution_type={}".format(repr(self._execution_type)))
        fields.append("assignee={}".format(repr(self._assignee)))
        fields.append("cloned_from={}".format(repr(self._cloned_from)))
        fields.append("creation_origin={}".format(repr(self._creation_origin)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("execution_origin={}".format(repr(self._execution_origin)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("outputs={}".format(repr(self._outputs)))
        fields.append("scheduled_on={}".format(repr(self._scheduled_on)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("workflow_task_group={}".format(repr(self._workflow_task_group)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTask({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        execution_type: Union[Unset, int] = UNSET
        if not isinstance(self._execution_type, Unset):
            execution_type = self._execution_type.value

        assignee: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._assignee, Unset):
            assignee = self._assignee.to_dict() if self._assignee else None

        cloned_from: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._cloned_from, Unset):
            cloned_from = self._cloned_from.to_dict() if self._cloned_from else None

        creation_origin: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creation_origin, Unset):
            creation_origin = self._creation_origin.to_dict()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        execution_origin: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._execution_origin, Unset):
            execution_origin = self._execution_origin.to_dict() if self._execution_origin else None

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        outputs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._outputs, Unset):
            outputs = []
            for outputs_item_data in self._outputs:
                outputs_item = outputs_item_data.to_dict()

                outputs.append(outputs_item)

        scheduled_on: Union[Unset, None, str] = UNSET
        if not isinstance(self._scheduled_on, Unset):
            scheduled_on = self._scheduled_on.isoformat() if self._scheduled_on else None

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.to_dict()

        web_url = self._web_url
        workflow_task_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._workflow_task_group, Unset):
            workflow_task_group = self._workflow_task_group.to_dict()

        display_id = self._display_id
        id = self._id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if execution_type is not UNSET:
            field_dict["executionType"] = execution_type
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if cloned_from is not UNSET:
            field_dict["clonedFrom"] = cloned_from
        if creation_origin is not UNSET:
            field_dict["creationOrigin"] = creation_origin
        if creator is not UNSET:
            field_dict["creator"] = creator
        if execution_origin is not UNSET:
            field_dict["executionOrigin"] = execution_origin
        if fields is not UNSET:
            field_dict["fields"] = fields
        if outputs is not UNSET:
            field_dict["outputs"] = outputs
        if scheduled_on is not UNSET:
            field_dict["scheduledOn"] = scheduled_on
        if status is not UNSET:
            field_dict["status"] = status
        if web_url is not UNSET:
            field_dict["webURL"] = web_url
        if workflow_task_group is not UNSET:
            field_dict["workflowTaskGroup"] = workflow_task_group
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_execution_type() -> Union[Unset, WorkflowTaskExecutionType]:
            execution_type = UNSET
            _execution_type = d.pop("executionType")
            if _execution_type is not None and _execution_type is not UNSET:
                try:
                    execution_type = WorkflowTaskExecutionType(_execution_type)
                except ValueError:
                    execution_type = WorkflowTaskExecutionType.of_unknown(_execution_type)

            return execution_type

        try:
            execution_type = get_execution_type()
        except KeyError:
            if strict:
                raise
            execution_type = cast(Union[Unset, WorkflowTaskExecutionType], UNSET)

        def get_assignee() -> Union[Unset, None, UserSummary]:
            assignee = None
            _assignee = d.pop("assignee")

            if _assignee is not None and not isinstance(_assignee, Unset):
                assignee = UserSummary.from_dict(_assignee)

            return assignee

        try:
            assignee = get_assignee()
        except KeyError:
            if strict:
                raise
            assignee = cast(Union[Unset, None, UserSummary], UNSET)

        def get_cloned_from() -> Union[Unset, None, WorkflowTaskSummary]:
            cloned_from = None
            _cloned_from = d.pop("clonedFrom")

            if _cloned_from is not None and not isinstance(_cloned_from, Unset):
                cloned_from = WorkflowTaskSummary.from_dict(_cloned_from)

            return cloned_from

        try:
            cloned_from = get_cloned_from()
        except KeyError:
            if strict:
                raise
            cloned_from = cast(Union[Unset, None, WorkflowTaskSummary], UNSET)

        def get_creation_origin() -> Union[Unset, CreationOrigin]:
            creation_origin: Union[Unset, Union[Unset, CreationOrigin]] = UNSET
            _creation_origin = d.pop("creationOrigin")

            if not isinstance(_creation_origin, Unset):
                creation_origin = CreationOrigin.from_dict(_creation_origin)

            return creation_origin

        try:
            creation_origin = get_creation_origin()
        except KeyError:
            if strict:
                raise
            creation_origin = cast(Union[Unset, CreationOrigin], UNSET)

        def get_creator() -> Union[Unset, UserSummary]:
            creator: Union[Unset, Union[Unset, UserSummary]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = UserSummary.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, UserSummary], UNSET)

        def get_execution_origin() -> Union[Unset, None, WorkflowTaskExecutionOrigin]:
            execution_origin = None
            _execution_origin = d.pop("executionOrigin")

            if _execution_origin is not None and not isinstance(_execution_origin, Unset):
                execution_origin = WorkflowTaskExecutionOrigin.from_dict(_execution_origin)

            return execution_origin

        try:
            execution_origin = get_execution_origin()
        except KeyError:
            if strict:
                raise
            execution_origin = cast(Union[Unset, None, WorkflowTaskExecutionOrigin], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_outputs() -> Union[Unset, List[WorkflowOutputSummary]]:
            outputs = []
            _outputs = d.pop("outputs")
            for outputs_item_data in _outputs or []:
                outputs_item = WorkflowOutputSummary.from_dict(outputs_item_data, strict=False)

                outputs.append(outputs_item)

            return outputs

        try:
            outputs = get_outputs()
        except KeyError:
            if strict:
                raise
            outputs = cast(Union[Unset, List[WorkflowOutputSummary]], UNSET)

        def get_scheduled_on() -> Union[Unset, None, datetime.date]:
            scheduled_on: Union[Unset, None, datetime.date] = UNSET
            _scheduled_on = d.pop("scheduledOn")
            if _scheduled_on is not None and not isinstance(_scheduled_on, Unset):
                scheduled_on = isoparse(cast(str, _scheduled_on)).date()

            return scheduled_on

        try:
            scheduled_on = get_scheduled_on()
        except KeyError:
            if strict:
                raise
            scheduled_on = cast(Union[Unset, None, datetime.date], UNSET)

        def get_status() -> Union[Unset, WorkflowTaskStatus]:
            status: Union[Unset, Union[Unset, WorkflowTaskStatus]] = UNSET
            _status = d.pop("status")

            if not isinstance(_status, Unset):
                status = WorkflowTaskStatus.from_dict(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, WorkflowTaskStatus], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_workflow_task_group() -> Union[Unset, WorkflowTaskGroupSummary]:
            workflow_task_group: Union[Unset, Union[Unset, WorkflowTaskGroupSummary]] = UNSET
            _workflow_task_group = d.pop("workflowTaskGroup")

            if not isinstance(_workflow_task_group, Unset):
                workflow_task_group = WorkflowTaskGroupSummary.from_dict(_workflow_task_group)

            return workflow_task_group

        try:
            workflow_task_group = get_workflow_task_group()
        except KeyError:
            if strict:
                raise
            workflow_task_group = cast(Union[Unset, WorkflowTaskGroupSummary], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        workflow_task = cls(
            execution_type=execution_type,
            assignee=assignee,
            cloned_from=cloned_from,
            creation_origin=creation_origin,
            creator=creator,
            execution_origin=execution_origin,
            fields=fields,
            outputs=outputs,
            scheduled_on=scheduled_on,
            status=status,
            web_url=web_url,
            workflow_task_group=workflow_task_group,
            display_id=display_id,
            id=id,
        )

        workflow_task.additional_properties = d
        return workflow_task

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def execution_type(self) -> WorkflowTaskExecutionType:
        """ The method by which the task of the workflow is executed """
        if isinstance(self._execution_type, Unset):
            raise NotPresentError(self, "execution_type")
        return self._execution_type

    @execution_type.setter
    def execution_type(self, value: WorkflowTaskExecutionType) -> None:
        self._execution_type = value

    @execution_type.deleter
    def execution_type(self) -> None:
        self._execution_type = UNSET

    @property
    def assignee(self) -> Optional[UserSummary]:
        if isinstance(self._assignee, Unset):
            raise NotPresentError(self, "assignee")
        return self._assignee

    @assignee.setter
    def assignee(self, value: Optional[UserSummary]) -> None:
        self._assignee = value

    @assignee.deleter
    def assignee(self) -> None:
        self._assignee = UNSET

    @property
    def cloned_from(self) -> Optional[WorkflowTaskSummary]:
        if isinstance(self._cloned_from, Unset):
            raise NotPresentError(self, "cloned_from")
        return self._cloned_from

    @cloned_from.setter
    def cloned_from(self, value: Optional[WorkflowTaskSummary]) -> None:
        self._cloned_from = value

    @cloned_from.deleter
    def cloned_from(self) -> None:
        self._cloned_from = UNSET

    @property
    def creation_origin(self) -> CreationOrigin:
        if isinstance(self._creation_origin, Unset):
            raise NotPresentError(self, "creation_origin")
        return self._creation_origin

    @creation_origin.setter
    def creation_origin(self, value: CreationOrigin) -> None:
        self._creation_origin = value

    @creation_origin.deleter
    def creation_origin(self) -> None:
        self._creation_origin = UNSET

    @property
    def creator(self) -> UserSummary:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: UserSummary) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def execution_origin(self) -> Optional[WorkflowTaskExecutionOrigin]:
        """ The context into which a task was executed """
        if isinstance(self._execution_origin, Unset):
            raise NotPresentError(self, "execution_origin")
        return self._execution_origin

    @execution_origin.setter
    def execution_origin(self, value: Optional[WorkflowTaskExecutionOrigin]) -> None:
        self._execution_origin = value

    @execution_origin.deleter
    def execution_origin(self) -> None:
        self._execution_origin = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def outputs(self) -> List[WorkflowOutputSummary]:
        if isinstance(self._outputs, Unset):
            raise NotPresentError(self, "outputs")
        return self._outputs

    @outputs.setter
    def outputs(self, value: List[WorkflowOutputSummary]) -> None:
        self._outputs = value

    @outputs.deleter
    def outputs(self) -> None:
        self._outputs = UNSET

    @property
    def scheduled_on(self) -> Optional[datetime.date]:
        """ The date on which the task is scheduled to be executed """
        if isinstance(self._scheduled_on, Unset):
            raise NotPresentError(self, "scheduled_on")
        return self._scheduled_on

    @scheduled_on.setter
    def scheduled_on(self, value: Optional[datetime.date]) -> None:
        self._scheduled_on = value

    @scheduled_on.deleter
    def scheduled_on(self) -> None:
        self._scheduled_on = UNSET

    @property
    def status(self) -> WorkflowTaskStatus:
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: WorkflowTaskStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET

    @property
    def web_url(self) -> str:
        """ URL of the workflow task """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def workflow_task_group(self) -> WorkflowTaskGroupSummary:
        if isinstance(self._workflow_task_group, Unset):
            raise NotPresentError(self, "workflow_task_group")
        return self._workflow_task_group

    @workflow_task_group.setter
    def workflow_task_group(self, value: WorkflowTaskGroupSummary) -> None:
        self._workflow_task_group = value

    @workflow_task_group.deleter
    def workflow_task_group(self) -> None:
        self._workflow_task_group = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the workflow task """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def id(self) -> str:
        """ The ID of the workflow task """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.policy_statement import PolicyStatement
from ..types import UNSET, Unset

T = TypeVar("T", bound="AccessPolicy")


@attr.s(auto_attribs=True, repr=False)
class AccessPolicy:
    """  """

    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _statements: Union[Unset, List[PolicyStatement]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("statements={}".format(repr(self._statements)))
        return "AccessPolicy({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        name = self._name
        statements: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._statements, Unset):
            statements = []
            for statements_item_data in self._statements:
                statements_item = statements_item_data.to_dict()

                statements.append(statements_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if statements is not UNSET:
            field_dict["statements"] = statements

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_statements() -> Union[Unset, List[PolicyStatement]]:
            statements = []
            _statements = d.pop("statements")
            for statements_item_data in _statements or []:
                statements_item = PolicyStatement.from_dict(statements_item_data, strict=False)

                statements.append(statements_item)

            return statements

        try:
            statements = get_statements()
        except KeyError:
            if strict:
                raise
            statements = cast(Union[Unset, List[PolicyStatement]], UNSET)

        access_policy = cls(
            id=id,
            name=name,
            statements=statements,
        )

        return access_policy

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def statements(self) -> List[PolicyStatement]:
        if isinstance(self._statements, Unset):
            raise NotPresentError(self, "statements")
        return self._statements

    @statements.setter
    def statements(self, value: List[PolicyStatement]) -> None:
        self._statements = value

    @statements.deleter
    def statements(self) -> None:
        self._statements = UNSET



from typing import Any, Dict, List, Optional, Type, TypeVar

import attr

from ..models.custom_field import CustomField

T = TypeVar("T", bound="CustomFields")


@attr.s(auto_attribs=True, repr=False)
class CustomFields:
    """  """

    additional_properties: Dict[str, CustomField] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "CustomFields({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        custom_fields = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = CustomField.from_dict(prop_dict, strict=False)

            additional_properties[prop_name] = additional_property

        custom_fields.additional_properties = additional_properties
        return custom_fields

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> CustomField:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: CustomField) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[CustomField]:
        return self.additional_properties.get(key, default)



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_task_status import WorkflowTaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskStatusLifecycleTransition")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskStatusLifecycleTransition:
    """  """

    _from_: Union[Unset, WorkflowTaskStatus] = UNSET
    _to: Union[Unset, WorkflowTaskStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("from_={}".format(repr(self._from_)))
        fields.append("to={}".format(repr(self._to)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskStatusLifecycleTransition({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        from_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._from_, Unset):
            from_ = self._from_.to_dict()

        to: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._to, Unset):
            to = self._to.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_from_() -> Union[Unset, WorkflowTaskStatus]:
            from_: Union[Unset, Union[Unset, WorkflowTaskStatus]] = UNSET
            _from_ = d.pop("from")

            if not isinstance(_from_, Unset):
                from_ = WorkflowTaskStatus.from_dict(_from_)

            return from_

        try:
            from_ = get_from_()
        except KeyError:
            if strict:
                raise
            from_ = cast(Union[Unset, WorkflowTaskStatus], UNSET)

        def get_to() -> Union[Unset, WorkflowTaskStatus]:
            to: Union[Unset, Union[Unset, WorkflowTaskStatus]] = UNSET
            _to = d.pop("to")

            if not isinstance(_to, Unset):
                to = WorkflowTaskStatus.from_dict(_to)

            return to

        try:
            to = get_to()
        except KeyError:
            if strict:
                raise
            to = cast(Union[Unset, WorkflowTaskStatus], UNSET)

        workflow_task_status_lifecycle_transition = cls(
            from_=from_,
            to=to,
        )

        workflow_task_status_lifecycle_transition.additional_properties = d
        return workflow_task_status_lifecycle_transition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def from_(self) -> WorkflowTaskStatus:
        if isinstance(self._from_, Unset):
            raise NotPresentError(self, "from_")
        return self._from_

    @from_.setter
    def from_(self, value: WorkflowTaskStatus) -> None:
        self._from_ = value

    @from_.deleter
    def from_(self) -> None:
        self._from_ = UNSET

    @property
    def to(self) -> WorkflowTaskStatus:
        if isinstance(self._to, Unset):
            raise NotPresentError(self, "to")
        return self._to

    @to.setter
    def to(self, value: WorkflowTaskStatus) -> None:
        self._to = value

    @to.deleter
    def to(self) -> None:
        self._to = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.free_form_text_form_field_canned_responses_behavior import (
    FreeFormTextFormFieldCannedResponsesBehavior,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="FreeFormTextFormFieldCannedResponses")


@attr.s(auto_attribs=True, repr=False)
class FreeFormTextFormFieldCannedResponses:
    """  """

    _behavior: Union[Unset, FreeFormTextFormFieldCannedResponsesBehavior] = UNSET
    _values: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("behavior={}".format(repr(self._behavior)))
        fields.append("values={}".format(repr(self._values)))
        return "FreeFormTextFormFieldCannedResponses({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        behavior: Union[Unset, int] = UNSET
        if not isinstance(self._behavior, Unset):
            behavior = self._behavior.value

        values: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._values, Unset):
            values = self._values

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if behavior is not UNSET:
            field_dict["behavior"] = behavior
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_behavior() -> Union[Unset, FreeFormTextFormFieldCannedResponsesBehavior]:
            behavior = UNSET
            _behavior = d.pop("behavior")
            if _behavior is not None and _behavior is not UNSET:
                try:
                    behavior = FreeFormTextFormFieldCannedResponsesBehavior(_behavior)
                except ValueError:
                    behavior = FreeFormTextFormFieldCannedResponsesBehavior.of_unknown(_behavior)

            return behavior

        try:
            behavior = get_behavior()
        except KeyError:
            if strict:
                raise
            behavior = cast(Union[Unset, FreeFormTextFormFieldCannedResponsesBehavior], UNSET)

        def get_values() -> Union[Unset, List[str]]:
            values = cast(List[str], d.pop("values"))

            return values

        try:
            values = get_values()
        except KeyError:
            if strict:
                raise
            values = cast(Union[Unset, List[str]], UNSET)

        free_form_text_form_field_canned_responses = cls(
            behavior=behavior,
            values=values,
        )

        return free_form_text_form_field_canned_responses

    @property
    def behavior(self) -> FreeFormTextFormFieldCannedResponsesBehavior:
        if isinstance(self._behavior, Unset):
            raise NotPresentError(self, "behavior")
        return self._behavior

    @behavior.setter
    def behavior(self, value: FreeFormTextFormFieldCannedResponsesBehavior) -> None:
        self._behavior = value

    @behavior.deleter
    def behavior(self) -> None:
        self._behavior = UNSET

    @property
    def values(self) -> List[str]:
        if isinstance(self._values, Unset):
            raise NotPresentError(self, "values")
        return self._values

    @values.setter
    def values(self, value: List[str]) -> None:
        self._values = value

    @values.deleter
    def values(self) -> None:
        self._values = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.box_schema import BoxSchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="BoxSchemasList")


@attr.s(auto_attribs=True, repr=False)
class BoxSchemasList:
    """  """

    _box_schemas: Union[Unset, List[BoxSchema]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("box_schemas={}".format(repr(self._box_schemas)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BoxSchemasList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        box_schemas: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._box_schemas, Unset):
            box_schemas = []
            for box_schemas_item_data in self._box_schemas:
                box_schemas_item = box_schemas_item_data.to_dict()

                box_schemas.append(box_schemas_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if box_schemas is not UNSET:
            field_dict["boxSchemas"] = box_schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_box_schemas() -> Union[Unset, List[BoxSchema]]:
            box_schemas = []
            _box_schemas = d.pop("boxSchemas")
            for box_schemas_item_data in _box_schemas or []:
                box_schemas_item = BoxSchema.from_dict(box_schemas_item_data, strict=False)

                box_schemas.append(box_schemas_item)

            return box_schemas

        try:
            box_schemas = get_box_schemas()
        except KeyError:
            if strict:
                raise
            box_schemas = cast(Union[Unset, List[BoxSchema]], UNSET)

        box_schemas_list = cls(
            box_schemas=box_schemas,
        )

        box_schemas_list.additional_properties = d
        return box_schemas_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def box_schemas(self) -> List[BoxSchema]:
        if isinstance(self._box_schemas, Unset):
            raise NotPresentError(self, "box_schemas")
        return self._box_schemas

    @box_schemas.setter
    def box_schemas(self, value: List[BoxSchema]) -> None:
        self._box_schemas = value

    @box_schemas.deleter
    def box_schemas(self) -> None:
        self._box_schemas = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.assay_results_paginated_list import AssayResultsPaginatedList
from ...models.list_assay_results_sort import ListAssayResultsSort
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    schema_id: str,
    created_atlt: Union[Unset, str] = UNSET,
    created_atgt: Union[Unset, str] = UNSET,
    created_atlte: Union[Unset, str] = UNSET,
    created_atgte: Union[Unset, str] = UNSET,
    min_created_time: Union[Unset, int] = UNSET,
    max_created_time: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListAssayResultsSort] = ListAssayResultsSort.CREATEDATASC,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    entity_ids: Union[Unset, str] = UNSET,
    storage_ids: Union[Unset, str] = UNSET,
    assay_run_ids: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    modified_atlt: Union[Unset, str] = UNSET,
    modified_atgt: Union[Unset, str] = UNSET,
    modified_atlte: Union[Unset, str] = UNSET,
    modified_atgte: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/assay-results".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_sort: Union[Unset, int] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    params: Dict[str, Any] = {
        "schemaId": schema_id,
    }
    if not isinstance(created_atlt, Unset) and created_atlt is not None:
        params["createdAt.lt"] = created_atlt
    if not isinstance(created_atgt, Unset) and created_atgt is not None:
        params["createdAt.gt"] = created_atgt
    if not isinstance(created_atlte, Unset) and created_atlte is not None:
        params["createdAt.lte"] = created_atlte
    if not isinstance(created_atgte, Unset) and created_atgte is not None:
        params["createdAt.gte"] = created_atgte
    if not isinstance(min_created_time, Unset) and min_created_time is not None:
        params["minCreatedTime"] = min_created_time
    if not isinstance(max_created_time, Unset) and max_created_time is not None:
        params["maxCreatedTime"] = max_created_time
    if not isinstance(json_sort, Unset) and json_sort is not None:
        params["sort"] = json_sort
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(entity_ids, Unset) and entity_ids is not None:
        params["entityIds"] = entity_ids
    if not isinstance(storage_ids, Unset) and storage_ids is not None:
        params["storageIds"] = storage_ids
    if not isinstance(assay_run_ids, Unset) and assay_run_ids is not None:
        params["assayRunIds"] = assay_run_ids
    if not isinstance(ids, Unset) and ids is not None:
        params["ids"] = ids
    if not isinstance(modified_atlt, Unset) and modified_atlt is not None:
        params["modifiedAt.lt"] = modified_atlt
    if not isinstance(modified_atgt, Unset) and modified_atgt is not None:
        params["modifiedAt.gt"] = modified_atgt
    if not isinstance(modified_atlte, Unset) and modified_atlte is not None:
        params["modifiedAt.lte"] = modified_atlte
    if not isinstance(modified_atgte, Unset) and modified_atgte is not None:
        params["modifiedAt.gte"] = modified_atgte
    if not isinstance(archive_reason, Unset) and archive_reason is not None:
        params["archiveReason"] = archive_reason

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[AssayResultsPaginatedList]:
    if response.status_code == 200:
        response_200 = AssayResultsPaginatedList.from_dict(response.json(), strict=False)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[AssayResultsPaginatedList]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    schema_id: str,
    created_atlt: Union[Unset, str] = UNSET,
    created_atgt: Union[Unset, str] = UNSET,
    created_atlte: Union[Unset, str] = UNSET,
    created_atgte: Union[Unset, str] = UNSET,
    min_created_time: Union[Unset, int] = UNSET,
    max_created_time: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListAssayResultsSort] = ListAssayResultsSort.CREATEDATASC,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    entity_ids: Union[Unset, str] = UNSET,
    storage_ids: Union[Unset, str] = UNSET,
    assay_run_ids: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    modified_atlt: Union[Unset, str] = UNSET,
    modified_atgt: Union[Unset, str] = UNSET,
    modified_atlte: Union[Unset, str] = UNSET,
    modified_atgte: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[AssayResultsPaginatedList]:
    kwargs = _get_kwargs(
        client=client,
        schema_id=schema_id,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        min_created_time=min_created_time,
        max_created_time=max_created_time,
        sort=sort,
        next_token=next_token,
        page_size=page_size,
        entity_ids=entity_ids,
        storage_ids=storage_ids,
        assay_run_ids=assay_run_ids,
        ids=ids,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        archive_reason=archive_reason,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    schema_id: str,
    created_atlt: Union[Unset, str] = UNSET,
    created_atgt: Union[Unset, str] = UNSET,
    created_atlte: Union[Unset, str] = UNSET,
    created_atgte: Union[Unset, str] = UNSET,
    min_created_time: Union[Unset, int] = UNSET,
    max_created_time: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListAssayResultsSort] = ListAssayResultsSort.CREATEDATASC,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    entity_ids: Union[Unset, str] = UNSET,
    storage_ids: Union[Unset, str] = UNSET,
    assay_run_ids: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    modified_atlt: Union[Unset, str] = UNSET,
    modified_atgt: Union[Unset, str] = UNSET,
    modified_atlte: Union[Unset, str] = UNSET,
    modified_atgte: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[AssayResultsPaginatedList]:
    """ List results """

    return sync_detailed(
        client=client,
        schema_id=schema_id,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        min_created_time=min_created_time,
        max_created_time=max_created_time,
        sort=sort,
        next_token=next_token,
        page_size=page_size,
        entity_ids=entity_ids,
        storage_ids=storage_ids,
        assay_run_ids=assay_run_ids,
        ids=ids,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        archive_reason=archive_reason,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    schema_id: str,
    created_atlt: Union[Unset, str] = UNSET,
    created_atgt: Union[Unset, str] = UNSET,
    created_atlte: Union[Unset, str] = UNSET,
    created_atgte: Union[Unset, str] = UNSET,
    min_created_time: Union[Unset, int] = UNSET,
    max_created_time: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListAssayResultsSort] = ListAssayResultsSort.CREATEDATASC,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    entity_ids: Union[Unset, str] = UNSET,
    storage_ids: Union[Unset, str] = UNSET,
    assay_run_ids: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    modified_atlt: Union[Unset, str] = UNSET,
    modified_atgt: Union[Unset, str] = UNSET,
    modified_atlte: Union[Unset, str] = UNSET,
    modified_atgte: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[AssayResultsPaginatedList]:
    kwargs = _get_kwargs(
        client=client,
        schema_id=schema_id,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        min_created_time=min_created_time,
        max_created_time=max_created_time,
        sort=sort,
        next_token=next_token,
        page_size=page_size,
        entity_ids=entity_ids,
        storage_ids=storage_ids,
        assay_run_ids=assay_run_ids,
        ids=ids,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        archive_reason=archive_reason,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    schema_id: str,
    created_atlt: Union[Unset, str] = UNSET,
    created_atgt: Union[Unset, str] = UNSET,
    created_atlte: Union[Unset, str] = UNSET,
    created_atgte: Union[Unset, str] = UNSET,
    min_created_time: Union[Unset, int] = UNSET,
    max_created_time: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListAssayResultsSort] = ListAssayResultsSort.CREATEDATASC,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    entity_ids: Union[Unset, str] = UNSET,
    storage_ids: Union[Unset, str] = UNSET,
    assay_run_ids: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    modified_atlt: Union[Unset, str] = UNSET,
    modified_atgt: Union[Unset, str] = UNSET,
    modified_atlte: Union[Unset, str] = UNSET,
    modified_atgte: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[AssayResultsPaginatedList]:
    """ List results """

    return (
        await asyncio_detailed(
            client=client,
            schema_id=schema_id,
            created_atlt=created_atlt,
            created_atgt=created_atgt,
            created_atlte=created_atlte,
            created_atgte=created_atgte,
            min_created_time=min_created_time,
            max_created_time=max_created_time,
            sort=sort,
            next_token=next_token,
            page_size=page_size,
            entity_ids=entity_ids,
            storage_ids=storage_ids,
            assay_run_ids=assay_run_ids,
            ids=ids,
            modified_atlt=modified_atlt,
            modified_atgt=modified_atgt,
            modified_atlte=modified_atlte,
            modified_atgte=modified_atgte,
            archive_reason=archive_reason,
        )
    ).parsed



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['benchling_api_client',
 'benchling_api_client.api',
 'benchling_api_client.api.aa_sequences',
 'benchling_api_client.api.apps',
 'benchling_api_client.api.assay_results',
 'benchling_api_client.api.assay_runs',
 'benchling_api_client.api.authentication',
 'benchling_api_client.api.batches',
 'benchling_api_client.api.blobs',
 'benchling_api_client.api.boxes',
 'benchling_api_client.api.containers',
 'benchling_api_client.api.custom_entities',
 'benchling_api_client.api.dna_alignments',
 'benchling_api_client.api.dna_oligos',
 'benchling_api_client.api.dna_sequences',
 'benchling_api_client.api.dropdowns',
 'benchling_api_client.api.entries',
 'benchling_api_client.api.events',
 'benchling_api_client.api.exports',
 'benchling_api_client.api.feature_libraries',
 'benchling_api_client.api.folders',
 'benchling_api_client.api.inventory',
 'benchling_api_client.api.lab_automation',
 'benchling_api_client.api.label_templates',
 'benchling_api_client.api.legacy_workflows',
 'benchling_api_client.api.legacy_workflows_deprecated',
 'benchling_api_client.api.locations',
 'benchling_api_client.api.mixtures',
 'benchling_api_client.api.molecules',
 'benchling_api_client.api.nucleotide_alignments',
 'benchling_api_client.api.oligos',
 'benchling_api_client.api.organizations',
 'benchling_api_client.api.plates',
 'benchling_api_client.api.printers',
 'benchling_api_client.api.projects',
 'benchling_api_client.api.registry',
 'benchling_api_client.api.requests',
 'benchling_api_client.api.rna_oligos',
 'benchling_api_client.api.rna_sequences',
 'benchling_api_client.api.schemas',
 'benchling_api_client.api.tasks',
 'benchling_api_client.api.teams',
 'benchling_api_client.api.users',
 'benchling_api_client.api.warehouse',
 'benchling_api_client.api.workflow_outputs',
 'benchling_api_client.api.workflow_task_groups',
 'benchling_api_client.api.workflow_tasks',
 'benchling_api_client.models',
 'benchling_api_client.v2',
 'benchling_api_client.v2.alpha',
 'benchling_api_client.v2.alpha.api',
 'benchling_api_client.v2.alpha.api.aa_sequences',
 'benchling_api_client.v2.alpha.api.access_policies',
 'benchling_api_client.v2.alpha.api.analyses',
 'benchling_api_client.v2.alpha.api.apps',
 'benchling_api_client.v2.alpha.api.codon_usage_tables',
 'benchling_api_client.v2.alpha.api.dna_oligos',
 'benchling_api_client.v2.alpha.api.dna_sequences',
 'benchling_api_client.v2.alpha.api.entries',
 'benchling_api_client.v2.alpha.api.enzymes',
 'benchling_api_client.v2.alpha.api.forms',
 'benchling_api_client.v2.alpha.api.legacy_workflows',
 'benchling_api_client.v2.alpha.api.projects',
 'benchling_api_client.v2.alpha.api.results',
 'benchling_api_client.v2.alpha.api.rna_oligos',
 'benchling_api_client.v2.alpha.api.schemas',
 'benchling_api_client.v2.alpha.api.workflow_flowchart_config_versions',
 'benchling_api_client.v2.alpha.api.workflow_flowcharts',
 'benchling_api_client.v2.alpha.api.workflow_outputs',
 'benchling_api_client.v2.alpha.api.workflow_task_groups',
 'benchling_api_client.v2.alpha.api.workflow_tasks',
 'benchling_api_client.v2.alpha.api.worklists',
 'benchling_api_client.v2.alpha.models',
 'benchling_api_client.v2.auth',
 'benchling_api_client.v2.beta',
 'benchling_api_client.v2.beta.api',
 'benchling_api_client.v2.beta.api.aa_sequences',
 'benchling_api_client.v2.beta.api.access_policies',
 'benchling_api_client.v2.beta.api.apps',
 'benchling_api_client.v2.beta.api.custom_entities',
 'benchling_api_client.v2.beta.api.dna_oligos',
 'benchling_api_client.v2.beta.api.dna_sequences',
 'benchling_api_client.v2.beta.api.entities',
 'benchling_api_client.v2.beta.api.entries',
 'benchling_api_client.v2.beta.api.molecules',
 'benchling_api_client.v2.beta.api.organizations',
 'benchling_api_client.v2.beta.api.projects',
 'benchling_api_client.v2.beta.api.rna_oligos',
 'benchling_api_client.v2.beta.api.teams',
 'benchling_api_client.v2.beta.api.users',
 'benchling_api_client.v2.beta.api.worklists',
 'benchling_api_client.v2.beta.models',
 'benchling_api_client.v2.stable',
 'benchling_api_client.v2.stable.api',
 'benchling_api_client.v2.stable.api.aa_sequences',
 'benchling_api_client.v2.stable.api.apps',
 'benchling_api_client.v2.stable.api.assay_results',
 'benchling_api_client.v2.stable.api.assay_runs',
 'benchling_api_client.v2.stable.api.authentication',
 'benchling_api_client.v2.stable.api.batches',
 'benchling_api_client.v2.stable.api.blobs',
 'benchling_api_client.v2.stable.api.boxes',
 'benchling_api_client.v2.stable.api.containers',
 'benchling_api_client.v2.stable.api.custom_entities',
 'benchling_api_client.v2.stable.api.dna_alignments',
 'benchling_api_client.v2.stable.api.dna_oligos',
 'benchling_api_client.v2.stable.api.dna_sequences',
 'benchling_api_client.v2.stable.api.dropdowns',
 'benchling_api_client.v2.stable.api.entries',
 'benchling_api_client.v2.stable.api.events',
 'benchling_api_client.v2.stable.api.exports',
 'benchling_api_client.v2.stable.api.feature_libraries',
 'benchling_api_client.v2.stable.api.folders',
 'benchling_api_client.v2.stable.api.inventory',
 'benchling_api_client.v2.stable.api.lab_automation',
 'benchling_api_client.v2.stable.api.label_templates',
 'benchling_api_client.v2.stable.api.legacy_workflows',
 'benchling_api_client.v2.stable.api.legacy_workflows_deprecated',
 'benchling_api_client.v2.stable.api.locations',
 'benchling_api_client.v2.stable.api.mixtures',
 'benchling_api_client.v2.stable.api.molecules',
 'benchling_api_client.v2.stable.api.nucleotide_alignments',
 'benchling_api_client.v2.stable.api.oligos',
 'benchling_api_client.v2.stable.api.organizations',
 'benchling_api_client.v2.stable.api.plates',
 'benchling_api_client.v2.stable.api.printers',
 'benchling_api_client.v2.stable.api.projects',
 'benchling_api_client.v2.stable.api.registry',
 'benchling_api_client.v2.stable.api.requests',
 'benchling_api_client.v2.stable.api.rna_oligos',
 'benchling_api_client.v2.stable.api.rna_sequences',
 'benchling_api_client.v2.stable.api.schemas',
 'benchling_api_client.v2.stable.api.tasks',
 'benchling_api_client.v2.stable.api.teams',
 'benchling_api_client.v2.stable.api.users',
 'benchling_api_client.v2.stable.api.warehouse',
 'benchling_api_client.v2.stable.api.workflow_outputs',
 'benchling_api_client.v2.stable.api.workflow_task_groups',
 'benchling_api_client.v2.stable.api.workflow_tasks',
 'benchling_api_client.v2.stable.models',
 'benchling_api_client.webhooks',
 'benchling_api_client.webhooks.v0',
 'benchling_api_client.webhooks.v0.alpha',
 'benchling_api_client.webhooks.v0.alpha.api',
 'benchling_api_client.webhooks.v0.alpha.models',
 'benchling_api_client.webhooks.v0.beta',
 'benchling_api_client.webhooks.v0.beta.api',
 'benchling_api_client.webhooks.v0.beta.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<23.0',
 'backoff>=1.10.0,<2.0.0',
 'dataclasses-json>=0.5.2,<0.6.0',
 'httpx>=0.23.0',
 'python-dateutil>=2.8.0,<3.0.0',
 'typing-extensions>=3.7.4,<5.0']

setup_kwargs = {
    'name': 'benchling-api-client',
    'version': '2.0.110',
    'description': 'Autogenerated Python client from OpenAPI Python Client generator',
    'long_description': "# benchling-api-client\nA client generated from Benchling's OpenAPI definition files, using openapi-python-client.\n\nRather than using this package directly, we recommend using the\n[Benchling SDK](https://pypi.org/project/benchling-sdk/), which has extra scaffolding to make some endpoints easier to\nuse, and has been released to general availability.",
    'author': 'Benchling Support',
    'author_email': 'support@benchling.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, cast, Dict, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.resource_dependency_types import ResourceDependencyTypes
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResourceDependency")


@attr.s(auto_attribs=True, repr=False)
class ResourceDependency:
    """  """

    _type: ResourceDependencyTypes
    _name: str
    _required_config: Union[Unset, bool] = False
    _description: Union[Unset, None, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("type={}".format(repr(self._type)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("required_config={}".format(repr(self._required_config)))
        fields.append("description={}".format(repr(self._description)))
        return "ResourceDependency({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        type = self._type.value

        name = self._name
        required_config = self._required_config
        description = self._description

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if required_config is not UNSET:
            field_dict["requiredConfig"] = required_config
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_type() -> ResourceDependencyTypes:
            _type = d.pop("type")
            try:
                type = ResourceDependencyTypes(_type)
            except ValueError:
                type = ResourceDependencyTypes.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(ResourceDependencyTypes, UNSET)

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_required_config() -> Union[Unset, bool]:
            required_config = d.pop("requiredConfig")
            return required_config

        try:
            required_config = get_required_config()
        except KeyError:
            if strict:
                raise
            required_config = cast(Union[Unset, bool], UNSET)

        def get_description() -> Union[Unset, None, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, None, str], UNSET)

        resource_dependency = cls(
            type=type,
            name=name,
            required_config=required_config,
            description=description,
        )

        return resource_dependency

    @property
    def type(self) -> ResourceDependencyTypes:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: ResourceDependencyTypes) -> None:
        self._type = value

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def required_config(self) -> bool:
        if isinstance(self._required_config, Unset):
            raise NotPresentError(self, "required_config")
        return self._required_config

    @required_config.setter
    def required_config(self, value: bool) -> None:
        self._required_config = value

    @required_config.deleter
    def required_config(self) -> None:
        self._required_config = UNSET

    @property
    def description(self) -> Optional[str]:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: Optional[str]) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileIds")


@attr.s(auto_attribs=True, repr=False)
class FileIds:
    """ Up to 25 files may be associated with a step data """

    _file_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("file_ids={}".format(repr(self._file_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "FileIds({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        file_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._file_ids, Unset):
            file_ids = self._file_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if file_ids is not UNSET:
            field_dict["fileIds"] = file_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_file_ids() -> Union[Unset, List[str]]:
            file_ids = cast(List[str], d.pop("fileIds"))

            return file_ids

        try:
            file_ids = get_file_ids()
        except KeyError:
            if strict:
                raise
            file_ids = cast(Union[Unset, List[str]], UNSET)

        file_ids = cls(
            file_ids=file_ids,
        )

        file_ids.additional_properties = d
        return file_ids

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def file_ids(self) -> List[str]:
        if isinstance(self._file_ids, Unset):
            raise NotPresentError(self, "file_ids")
        return self._file_ids

    @file_ids.setter
    def file_ids(self, value: List[str]) -> None:
        self._file_ids = value

    @file_ids.deleter
    def file_ids(self) -> None:
        self._file_ids = UNSET
